"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const Listr = require("listr");
const localfs_1 = require("arc-apollo-codegen-core/lib/localfs");
const util_1 = require("util");
const engine_cli_1 = require("../../engine-cli");
const load_schema_1 = require("../../load-schema");
const fetch_schema_1 = require("../../fetch-schema");
class SchemaDownload extends command_1.Command {
    async run() {
        const { flags, args } = this.parse(SchemaDownload);
        const header = Array.isArray(flags.header) ? flags.header : [flags.header];
        const apiKey = flags.key;
        const pullFromEngine = !!apiKey && !flags.endpoint;
        const tasks = new Listr([
            load_schema_1.loadSchemaStep(this, pullFromEngine, apiKey, flags.engine, "Fetching local schema", async (ctx) => {
                ctx.schema = await fetch_schema_1.fetchSchema({
                    endpoint: flags.endpoint,
                    header: header.filter(x => Boolean(x)).map(x => JSON.parse(x)),
                });
            }),
            {
                title: `Saving schema to ${args.output}`,
                task: async (ctx) => {
                    await util_1.promisify(localfs_1.fs.writeFile)(args.output, JSON.stringify(ctx.schema));
                },
            },
        ]);
        return tasks.run();
    }
}
SchemaDownload.description = "Download the schema from your GraphQL endpoint.";
SchemaDownload.flags = Object.assign({ help: command_1.flags.help({
        char: "h",
        description: "Show command help",
    }), header: command_1.flags.string({
        multiple: true,
        parse: header => {
            const [key, value] = header.split(":");
            return JSON.stringify({ [key.trim()]: value.trim() });
        },
        description: "Additional headers to send to server for introspectionQuery",
    }), endpoint: command_1.flags.string({
        description: "The URL of the server to fetch the schema from or path to ./your/local/schema.graphql",
        default: "http://localhost:4000/graphql",
    }) }, engine_cli_1.engineFlags);
SchemaDownload.args = [
    {
        name: "output",
        description: "Path to write the introspection result to",
        required: true,
        default: "schema.json"
    }
];
exports.default = SchemaDownload;
//# sourceMappingURL=download.js.map