"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const localfs_1 = require("arc-apollo-codegen-core/lib/localfs");
const path = require("path");
const node_fetch_1 = require("node-fetch");
const graphql_tag_1 = require("graphql-tag");
const graphql_1 = require("graphql");
const apollo_link_1 = require("apollo-link");
const apollo_link_http_1 = require("apollo-link-http");
const loading_1 = require("arc-apollo-codegen-core/lib/loading");
const introspection = graphql_tag_1.default(graphql_1.introspectionQuery);
const loadSchemaFromString = async (schemaSource) => {
    const schema = graphql_1.buildSchema(schemaSource);
    const localSchema = await graphql_1.execute(schema, introspection);
    if (!localSchema || localSchema.errors)
        throw new Error(localSchema.errors.map(({ message }) => message).join("\n"));
    return localSchema.data.__schema;
};
exports.fromFile = async ({ endpoint }) => {
    try {
        const result = localfs_1.fs.readFileSync(endpoint, {
            encoding: "utf-8",
        });
        const ext = path.extname(endpoint);
        if (ext === ".json") {
            const parsed = JSON.parse(result);
            return parsed.data ? parsed.data.__schema :
                parsed.__schema ? parsed.__schema : parsed;
        }
        ;
        if (ext === ".graphql" || ext === ".graphqls" || ext === ".gql") {
            return await loadSchemaFromString(result);
        }
        if (ext === ".ts" || ext === ".tsx" || ext === ".js" || ext === ".jsx") {
            return await loadSchemaFromString(loading_1.extractDocumentFromJavascript(result));
        }
    }
    catch (e) {
        throw new Error(`Unable to read file ${endpoint}. ${e.message}`);
    }
};
exports.fetchSchema = async ({ endpoint, header }) => {
    if (!endpoint)
        throw new Error("No endpoint provided when fetching schema");
    if (localfs_1.fs.existsSync(endpoint))
        return exports.fromFile({ endpoint });
    const headers = header
        ? header.reduce((current, next) => (Object.assign({}, current, next)), {})
        : {};
    return apollo_link_1.toPromise(apollo_link_1.execute(apollo_link_http_1.createHttpLink({ uri: endpoint, fetch: node_fetch_1.default }), {
        query: introspection,
        context: { headers },
    })).then(({ data, errors }) => {
        if (errors)
            throw new Error(errors.map(({ message }) => message).join("\n"));
        return data.__schema;
    });
};
//# sourceMappingURL=fetch-schema.js.map