"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const engine_1 = require("./engine");
const apollo_link_1 = require("apollo-link");
const schema_1 = require("./operations/schema");
function loadSchemaStep(self, pullFromEngine, apiKey, engineFlag, notEngineTitle, notEngine) {
    return {
        title: pullFromEngine
            ? "Loading schema from Apollo Engine"
            : notEngineTitle,
        task: async (ctx) => {
            if (pullFromEngine) {
                const variables = {
                    id: engine_1.getIdFromKey(apiKey),
                    tag: "current",
                };
                const engineSchema = await apollo_link_1.toPromise(apollo_link_1.execute(engine_1.engineLink, {
                    query: schema_1.SCHEMA_QUERY,
                    variables,
                    context: Object.assign({ headers: { ["x-api-key"]: apiKey } }, (engineFlag && { uri: engineFlag })),
                }));
                if (engineSchema.data && engineSchema.data.service.schema) {
                    ctx.schema = engineSchema.data.service.schema.__schema;
                }
                else {
                    self.error("Failed to get schema from Apollo Engine");
                }
            }
            else {
                await notEngine(ctx);
            }
        },
    };
}
exports.loadSchemaStep = loadSchemaStep;
//# sourceMappingURL=load-schema.js.map